/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this
 * software and to discontinue the availability of this software. By using this
 * software, you agree to the additional terms and conditions found by
 * accessing the following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/*******************************************************************************
* File Name    : r_uart_lora.c
* Version      : 1.0
* Description  : UART driver for LoRaWANTM stack sample application.
******************************************************************************/
/*****************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 31.05.2019 1.00     First Release
******************************************************************************/

#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
#include "r_app_lora.h"
#include "dataflash.h"
#include "r_uart_lora.h"

uint8_t app_uart2_rxbuf[APP_UART2_RXBUF_SIZE] = {0};
uint8_t app_uart2_txbuf[APP_UART2_TXBUF_SIZE] = {0};
app_uart_rx_state_t app_uart2_rx_state = APP_UART_RX_IDLE;
app_uart_tx_state_t app_uart2_tx_state = APP_UART_TX_IDLE;

uint8_t uart_send_buf[50];

void app_uart2_init(void)
{
    R_UART2_Start();
    R_UART2_Receive(app_uart2_rxbuf, APP_UART2_RXBUF_SIZE);
}

void app_uart2_deinit(void)
{
    R_UART2_Stop();
}

void app_uart2_send(char * cmd)
{
    uint8_t len;

    memset(uart_send_buf, 0, sizeof(uart_send_buf));
    len = strlen(cmd);
    memcpy(uart_send_buf, cmd, len);

    if(APP_UART_TX_IDLE == app_uart2_tx_state)
    {
        app_uart2_tx_state = APP_UART_TX_RUNNING;
        R_UART2_Send((uint8_t *)uart_send_buf, len);
    }
}

void app_uart2_rx_done_callback(void)
{
    R_LORA_APP_ATResult_Generate(app_uart2_rxbuf[0]);

    R_UART2_Receive(app_uart2_rxbuf, APP_UART2_RXBUF_SIZE);
    app_uart2_rx_state = APP_UART_RX_DONE;
}

void app_uart2_tx_done_callback(void)
{
    app_uart2_tx_state = APP_UART_TX_DONE;
    app_uart2_update_state();
}

void app_uart2_rx_error_callback(void)
{
    app_uart2_rx_state = APP_UART_RX_ERROR;
}

void app_uart2_tx_error_callback(void)
{
    app_uart2_tx_state = APP_UART_TX_ERROR;
}

void app_uart2_rx_overrun_callback(void)
{
    app_uart2_rx_state = APP_UART_RX_OVERRUN;
}

void app_uart2_update_state(void)
{
    switch(app_uart2_rx_state)
    {
        case APP_UART_RX_DONE:
            app_uart2_rx_state = APP_UART_RX_IDLE;
            break;
        case APP_UART_RX_OVERRUN:
            R_UART2_Receive(app_uart2_rxbuf, APP_UART2_RXBUF_SIZE);
            app_uart2_rx_state = APP_UART_RX_IDLE;
        default:
            break;
    }
    
    switch(app_uart2_tx_state)
    {
        case APP_UART_TX_DONE:
            app_uart2_tx_state = APP_UART_TX_IDLE;
            break;
        default:
            break;
    }
}

bool app_uart2_can_sleep(void)
{
    if ((app_uart2_tx_state == APP_UART_TX_IDLE)
        && (app_uart2_rx_state == APP_UART_RX_IDLE))
    {
        return true;
    }
    return false;
}
