/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this
 * software and to discontinue the availability of this software. By using this
 * software, you agree to the additional terms and conditions found by
 * accessing the following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/*******************************************************************************
* File Name    : r_app_ble.c
* Version      : 1.0
* Description  : BLE controll application.
*                Connect with a smartphone to receive LoRa module setting parameters.
******************************************************************************/
/*****************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 31.05.2019 1.00     First Release
******************************************************************************/

/******************************************************************************
Includes
******************************************************************************/

#include "r_cg_macrodriver.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ll.h"
#include "rble.h"
#include "rble_app.h"
#include "prf_sel.h"
#include "timer.h"
#include "rscip_ext.h"
#include "dataflash.h"
#include "r_app_lora.h"
#include "vuarts.h"
#include "r_cg_userdefine.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

#ifndef char_t
typedef char            char_t;
#endif

/******************************************************************************
Macro definitions
******************************************************************************/

#define     RUN_COMMAND_QUEUE_SIZE          10
#define     ILLEGAL_CONNECTION_HANDLE       0xffff
#define     SEND_DATA_INTERVAL              100     // 100 * 10 = 1000ms(1s)

enum APP_KEY_FLAG_enum {
    APP_LTK,
    APP_ENCKEY,
    APP_IDKEY,
    APP_SIGNKEY
};

typedef struct {
    uint8_t                         Key_Set_Flags;
    struct RBLE_EVT_SM_Key_t        EncKey;
    struct RBLE_EVT_SM_Key_t        IrKey;
    struct RBLE_EVT_SM_Key_t        SignKey;
} APP_KEY_DATA;


/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

cb_func_t   cb_func;

extern uint8_t app_mode;

extern LoRa_Param_Flag_t    lpf;

extern uint8_t atcmd_buf[50];

void R_BLE_APP_Set_RunCmd(uint16_t cmd);
void R_BLE_APP_Send_Execution_Result(void);

/******************************************************************************
Private global variables and functions
******************************************************************************/

static void r_ble_app_rble_callback( RBLE_MODE mode );
static BOOL r_ble_app_RBLE_MODE_ACTIVE_mode( void );

/* --------------------------- Generic Access Profile --------------------------------*/
static RBLE_STATUS r_ble_app_GAP_RESET_CMD( void );
static RBLE_STATUS r_ble_app_GAP_BROADCAST_ENABLE_CMD( void );
static RBLE_STATUS r_ble_app_GAP_BROADCAST_DISABLE_CMD(void);
static RBLE_STATUS r_ble_app_GAP_SET_BONDING_MODE_CMD( void );
static RBLE_STATUS r_ble_app_GAP_SET_SECURITY_REQUEST_CMD( void );
static RBLE_STATUS r_ble_app_GAP_BONDING_RESPONSE_CMD( void );
static RBLE_STATUS r_ble_app_GAP_DISCONNECT_CMD( void );

static void r_ble_app_gap_callback( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_RESET_RESULT_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_CONNECTION_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_DISCONNECT_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_BONDING_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_SET_BONDING_MODE_COMP_event( RBLE_GAP_EVENT *event );
static BOOL r_ble_app_RBLE_GAP_EVENT_BONDING_REQ_IND_event( RBLE_GAP_EVENT *event );

/* ----------------------------- Security Manager ------------------------------------*/
static RBLE_STATUS r_ble_app_SM_CHK_BD_ADDR_REQ_RESP_CMD( void );
static RBLE_STATUS r_ble_app_SM_TK_REQ_RESP_CMD( void );
static RBLE_STATUS r_ble_app_SM_LTK_REQ_RESP_CMD( void );
static RBLE_STATUS r_ble_app_SM_IRK_REQ_RESP_CMD( void );

static void r_ble_app_sm_callback( RBLE_SM_EVENT *event );
static BOOL r_ble_app_RBLE_SM_TK_REQ_IND_event( RBLE_SM_EVENT *event );
static BOOL r_ble_app_RBLE_SM_LTK_REQ_IND_event( RBLE_SM_EVENT *event );
static BOOL r_ble_app_RBLE_SM_IRK_REQ_IND_event( RBLE_SM_EVENT *event );
static BOOL r_ble_app_RBLE_SM_KEY_IND_event( RBLE_SM_EVENT *event );
static BOOL r_ble_app_RBLE_SM_CHK_BD_ADDR_REQ_event( RBLE_SM_EVENT *event );

/* -------------------- Genral Purpose Communication(VUART) --------------------------*/
/* APIs. */
extern RBLE_STATUS RBLE_VUART_Server_Enable(uint16_t conhdl, RBLE_VUART_EVENT_HANDLER callback);
extern RBLE_STATUS RBLE_VUART_Server_Disable(void);
/* Call back function. */
static void r_ble_app_gpcs_callback(RBLE_VUART_EVENT *event);
static RBLE_STATUS r_ble_app_GPCS_Server_Enable_CMD(void);

static BOOL r_ble_app_compare_bdaddr(RBLE_BD_ADDR* a, RBLE_BD_ADDR* b);
static int_t r_ble_app_get_random(void);

static uint16_t         ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;
static BOOL             InBonding     = FALSE;
static RBLE_BD_ADDR     BondedAddr    ;
static uint8_t          SecStatus     ;
static APP_KEY_DATA     LocalKey      ;
static APP_KEY_DATA     RemoteKey     ;
static uint8_t          ConnectionIdx ;
static uint8_t          ConnectionType;
static RBLE_BD_ADDR     RemoteDevice  ;
static BOOL             FoundFlg      ;
static BOOL             CheckedIrkFlg ;

static uint16_t         RunCmd[RUN_COMMAND_QUEUE_SIZE];    // RBLE_APP_CMD_NO_enum value
static uint8_t          WrPtr     ;
static uint8_t          RdPtr     ;

static RBLE_BROADCAST_ENABLE_PARAM  g_advdata = {
    /* General Discoverable & Undirected Connectable */
    RBLE_GAP_GEN_DISCOVERABLE,
    RBLE_GAP_UND_CONNECTABLE ,
    {
        {
            0x30,                                       /* AdvIntervalMin */
            0x60,                                       /* AdvIntervalMax */
            RBLE_GAP_ADV_CONN_UNDIR,                    /* AdvType        */
            RBLE_ADDR_PUBLIC,                           /* OwnAddrType    */
            RBLE_ADDR_PUBLIC,                           /* DirectAddrType */
            { 0xbc, 0x9a, 0x78, 0x56, 0x34, 0x12 },     /* DirectAddr     */
            RBLE_ADV_ALL_CHANNELS,                      /* AdvChannelMap  */
            RBLE_ADV_ALLOW_SCAN_ANY_CON_ANY,            /* AdvFiltPolicy  */
            0x00,                                       /* Reserved       */
        },
        {
            /* Advertising Data Length */
            3 + 13,                 /* [Flags] + [Complete local name] */

            /* Advertising Data (max 31bytes) */
            2,                      /* Number of data */
                0x01,               /* AD Types: Flags */ 
                0x06,
            12,                     /* Number of data */
                0x09,               /* AD Types: Complete local name */
                'R', 'T', 'K', '5', 'R', 'L', 'G', '1', '4', '0', 'C', /* String */

            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        },
        {
            /* Scan Rsponse Data Length */
            0,
            /* Scan Response Data (max 31bytes) */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    }
};

/******************************************************************************
* Function Name: R_BLE_APP_Init
* Description  : Application Initialize
* Arguments    : void
* Return Value : TRUE  - if Command Queue is empty
*                FALSE - if it not empty
******************************************************************************/
BOOL R_BLE_APP_Init(void)
{
    BOOL        ret = TRUE;
    RBLE_STATUS ret_status;

    /* Initialize rBLE */
    ret_status = RBLE_Init(&r_ble_app_rble_callback);
    if ( RBLE_OK != ret_status )
    {
        ret = FALSE;
    }
    /* Initialize Run Command Queue Pointer, LCD String Buffer */
    WrPtr = 0;
    RdPtr = 0;

    /* Initialize Bonding Information Buffer */
    SecStatus      = RBLE_SMP_SEC_NONE;
    memset(&BondedAddr, 0, sizeof(RBLE_BD_ADDR));
    memset(&LocalKey  , 0, sizeof(APP_KEY_DATA));
    memset(&RemoteKey , 0, sizeof(APP_KEY_DATA));

    return( ret );
}

/******************************************************************************
* Function Name: R_BLE_APP_Run
* Description  : Application Command Execute
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_BLE_APP_Run(void)
{
    /* Loop until Run Command Queue is empty */
    while(RdPtr != WrPtr)
    {
        switch(RunCmd[RdPtr])
        {
            case RBLE_APP_CMD_NON:
                break;
            /* --------------------------- Generic Access Profile --------------------------------*/
            case GAP_RESET_CMD:
                r_ble_app_GAP_RESET_CMD();
                break;
            case GAP_BROADCAST_ENABLE_CMD:
                r_ble_app_GAP_BROADCAST_ENABLE_CMD();
                break;
            case GAP_BROADCAST_DISABLE_CMD:
                r_ble_app_GAP_BROADCAST_DISABLE_CMD();
                break;
            case GAP_SET_BONDING_MODE_CMD:
                r_ble_app_GAP_SET_BONDING_MODE_CMD();
                break;
            case GAP_SET_SECURITY_REQUEST_CMD:
                r_ble_app_GAP_SET_SECURITY_REQUEST_CMD();
                break;
            case GAP_BONDING_RESPONSE_CMD:
                r_ble_app_GAP_BONDING_RESPONSE_CMD();
                break;
            case GAP_DISCONNECT_CMD:
                r_ble_app_GAP_DISCONNECT_CMD();
                break;

            /* --------------------------- Security Manager -------------------------------------*/
            case SM_CHK_BD_ADDR_REQ_RESP_CMD:
                r_ble_app_SM_CHK_BD_ADDR_REQ_RESP_CMD();
                break;
            case SM_TK_REQ_RESP_CMD:
                r_ble_app_SM_TK_REQ_RESP_CMD();
                break;
            case SM_LTK_REQ_RESP_CMD:
                r_ble_app_SM_LTK_REQ_RESP_CMD();
                break;
            case SM_IRK_REQ_RESP_CMD:
                r_ble_app_SM_IRK_REQ_RESP_CMD();
                break;

            /* ------------------- Genral Purpose Communication(VUART) --------------------------*/
            case GPCS_Server_Enable_CMD:
                r_ble_app_GPCS_Server_Enable_CMD();
                break;
            case GPCS_Server_Disable_CMD:
                break;
            case GPCS_Server_Send_Indication_CMD:
                break;

            default:
                break;
        }
        
        /* Operate Run Command Queue Pointer as a Ring Buffer */
        GLOBAL_INT_DISABLE();
        RdPtr = (RdPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (RdPtr + 1);
        GLOBAL_INT_RESTORE();
    }
}

/******************************************************************************
* Function Name: R_BLE_APP_Is_Idle
* Description  : Application Idle Check
* Arguments    : void
* Return Value : TRUE  - if Command Queue is empty
*                FALSE - if it not empty
******************************************************************************/
BOOL R_BLE_APP_Is_Idle(void)
{
    BOOL    Idle = FALSE;

    /* Application Status is Idle when Command Queue is Empty */
    if(RdPtr == WrPtr)
    {
        Idle = TRUE;
    }

    return(Idle);
}

/******************************************************************************
* Function Name: R_BLE_APP_Set_RunCmd
* Description  : Set R_BLE_APP_Run command
* Arguments    : cmd - Command value, defined by RBLE_APP_CMD_NO_enum
* Return Value : void
******************************************************************************/
void R_BLE_APP_Set_RunCmd(uint16_t cmd)
{
    RunCmd[WrPtr] = cmd;

    /* Operate Run Command Queue Pointer as a Ring Buffer */
    GLOBAL_INT_DISABLE();
    WrPtr = (WrPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (WrPtr + 1);
    GLOBAL_INT_RESTORE();
}

/******************************************************************************
* Function Name: R_BLE_APP_Send_Execution_Result
* Description  : Send execution result of LoRaWAN AT command
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_BLE_APP_Send_Execution_Result(void)
{
    RBLE_VUART_Server_Send_Indication((const char *)atcmd_buf, 3);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_RESET_CMD
* Description  : Command processing of GAP_RESET_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_RESET_CMD(void)
{
    RBLE_STATUS ret_status;

    ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;
    ret_status = RBLE_GAP_Reset(&r_ble_app_gap_callback, &r_ble_app_sm_callback);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_BROADCAST_ENABLE_CMD
* Description  : Command processing of GAP_BROADCAST_ENABLE_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_BROADCAST_ENABLE_CMD(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Broadcast_Enable(g_advdata.disc_mode,
                                           g_advdata.conn_mode,
                                           &g_advdata.adv_info);
    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_BROADCAST_DISABLE_CMD
* Description  : Command processing of GAP_BROADCAST_ENABLE_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_BROADCAST_DISABLE_CMD(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Broadcast_Disable();

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_SET_BONDING_MODE_CMD
* Description  : Command processing of GAP_SET_BONDING_MODE_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_SET_BONDING_MODE_CMD(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Set_Bonding_Mode(RBLE_GAP_BONDABLE);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_SET_SECURITY_REQUEST_CMD
* Description  : Command processing of GAP_SET_SECURITY_REQUEST_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_SET_SECURITY_REQUEST_CMD(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Set_Security_Request(RBLE_GAP_SEC1_NOAUTH_PAIR_ENC);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_BONDING_RESPONSE_CMD
* Description  : Command processing of GAP_BONDING_RESPONSE_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_BONDING_RESPONSE_CMD(void)
{
    RBLE_STATUS ret_status;
    static RBLE_BOND_RESP_PARAM Param = {
        0x0000,                                         /* Connection handle           */
        RBLE_OK,                                        /* accept or reject bonding    */
        RBLE_IO_CAP_NO_INPUT_NO_OUTPUT,                 /* IO capabilities             */
        RBLE_OOB_AUTH_DATA_NOT_PRESENT,                 /* OOB flag                    */
        RBLE_AUTH_REQ_NO_MITM_BOND,                     /* Authentication Requirements */
        RBLE_SMP_MAX_ENC_SIZE_LEN,                      /* Encryption key size         */
        RBLE_KEY_DIST_ENCKEY | RBLE_KEY_DIST_IDKEY,     /* Initiator key distribution  */
        RBLE_KEY_DIST_ENCKEY,                           /* Responder key distribution  */
        0x00                                            /* Reserved                    */
    };

    Param.conhdl = ConnectionHdl;
    ret_status   = RBLE_GAP_Bonding_Response(&Param);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GAP_DISCONNECT_CMD
* Description  : Command processing of GAP_DISCONNECT_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GAP_DISCONNECT_CMD(void)
{
    RBLE_STATUS ret_status = FALSE;

    if (ConnectionHdl != ILLEGAL_CONNECTION_HANDLE)
    {
        ret_status = RBLE_GAP_Disconnect(ConnectionHdl);
    }

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_SM_CHK_BD_ADDR_REQ_RESP_CMD
* Description  : Command processing of SM_CHK_BD_ADDR_REQ_RESP_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_SM_CHK_BD_ADDR_REQ_RESP_CMD(void)
{
    RBLE_STATUS     ret_status;
    uint8_t         sec_status = RBLE_SMP_SEC_NONE;

    if (FoundFlg)
    {
        sec_status = SecStatus;
    }

    ret_status = RBLE_SM_Chk_Bd_Addr_Req_Resp(ConnectionIdx,
                                              ConnectionType,
                                              FoundFlg,
                                              sec_status,
                                              &BondedAddr
                                             );

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_SM_TK_REQ_RESP_CMD
* Description  : Command processing of SM_TK_REQ_RESP_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_SM_TK_REQ_RESP_CMD(void)
{
    RBLE_STATUS ret_status;
    static RBLE_KEY_VALUE tk = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00      /* TK(Temporary Key) */
    };
    uint32_t passkey;
    uint8_t* byteptr = (uint8_t*)&passkey;
    uint8_t  i;

    /* Generate PassKey (range:000,000 - 999,999) */
    passkey  = (uint32_t)r_ble_app_get_random();
    passkey |= (uint32_t)r_ble_app_get_random() << 16;
    passkey %= 1000000;

    /* Copy PassKey to TK(Temporary Key) */
    for(i = 0; i < sizeof(uint32_t); i++)
    {
        tk.key[RBLE_KEY_LEN - 1 - i] = byteptr[i];
    }
    ret_status = RBLE_SM_Tk_Req_Resp(ConnectionIdx, RBLE_OK, &tk);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_SM_LTK_REQ_RESP_CMD
* Description  : Command processing of SM_LTK_REQ_RESP_CMD
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_SM_LTK_REQ_RESP_CMD(void)
{
    RBLE_STATUS   ret_status;
    const uint8_t sec_prop = RBLE_SMP_KSEC_UNAUTH_NO_MITM;
    uint8_t       status   = RBLE_OK;
    int_t         i;

    /* Requested in Bonding Sequence (only in First Connection) */
    if (TRUE == InBonding)
    {
        /* Generate LTK(Long Term Key), Random Number, EDIV(Encrypted Diversifier)  */
        for(i = 0; i <  RBLE_RAND_NB_LEN; i++)
        {
            LocalKey.EncKey.nb.nb[i]   = (uint8_t)r_ble_app_get_random();
        }
        for(i = 0; i <  RBLE_KEY_LEN; i++)
        {
            LocalKey.EncKey.ltk.key[i] = (uint8_t)r_ble_app_get_random();
        }
        LocalKey.EncKey.ediv    = (uint16_t)r_ble_app_get_random();
        LocalKey.Key_Set_Flags |= (1 << APP_ENCKEY);
    }
    /* Requested in Encryption Start Sequence (in Connection After Bonding) */
    else
    {
        /* Tell latest LTK of Previous Connection */

        /* Check if latest LTK is valid */
        if(0u == (LocalKey.Key_Set_Flags & (1 << APP_ENCKEY)))
        {
            status = RBLE_ERR;
        }
    }
    ret_status = RBLE_SM_Ltk_Req_Resp(ConnectionIdx,
                                      status,
                                      sec_prop,
                                      LocalKey.EncKey.ediv,
                                      &LocalKey.EncKey.nb,
                                      &LocalKey.EncKey.ltk
                                     );
    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_SM_IRK_REQ_RESP_CMD
* Description  : Command processing of RBLE_GAP_EVENT_RESET_RESULT
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_SM_IRK_REQ_RESP_CMD(void)
{
    RBLE_STATUS   ret_status;
    uint8_t       status   = RBLE_ERR;
    static RBLE_KEY_VALUE irk = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  /* IRK(Identity Resolving Key) */
    };

    if ((!CheckedIrkFlg) && (0u != (RemoteKey.Key_Set_Flags & (1 << APP_IDKEY))))
    {
        CheckedIrkFlg = TRUE;
        irk           = RemoteKey.IrKey.ltk;
        status        = RBLE_OK;
    }

    ret_status = RBLE_SM_Irk_Req_Resp(ConnectionIdx, status, &BondedAddr, &irk, SecStatus);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_GPCS_Server_Enable_CMD
* Description  : Command processing of Genral purpose communication(VUART) server enable
* Arguments    : void
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static RBLE_STATUS r_ble_app_GPCS_Server_Enable_CMD(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_VUART_Server_Enable(ConnectionHdl, &r_ble_app_gpcs_callback);

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_rble_callback
* Description  : Callback function of RBLE
* Arguments    : mode - RBLE_MODE_enum
* Return Value : void
******************************************************************************/
static void r_ble_app_rble_callback(RBLE_MODE mode)
{
    switch(mode)
    {
        case RBLE_MODE_INITIALIZE:
            break;
        case RBLE_MODE_ACTIVE:
            r_ble_app_RBLE_MODE_ACTIVE_mode();
            break;
        case RBLE_MODE_RESET:
            break;
        case RBLE_MODE_SLEEP:
            break;
        case RBLE_MODE_ERROR:
            break;
        default:
            break;
    }
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_MODE_ACTIVE_mode
* Description  : Mode processing of RBLE
* Arguments    : void
* Return Value : TRUE
******************************************************************************/
static BOOL r_ble_app_RBLE_MODE_ACTIVE_mode(void)
{
    R_BLE_APP_Set_RunCmd(GAP_RESET_CMD);
    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_gap_callback
* Description  : Callback function of General Access Profile
* Arguments    : event - GAP Event Parameters
* Return Value : void
******************************************************************************/
static void r_ble_app_gap_callback(RBLE_GAP_EVENT *event)
{
    switch(event->type)
    {
        case RBLE_GAP_EVENT_RESET_RESULT:
            r_ble_app_RBLE_GAP_EVENT_RESET_RESULT_COMP_event(event);
            break;
        case RBLE_GAP_EVENT_SET_NAME_COMP:
            break;
        case RBLE_GAP_EVENT_OBSERVATION_ENABLE_COMP:
            break;
        case RBLE_GAP_EVENT_OBSERVATION_DISABLE_COMP:
            break;
        case RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP:
            r_ble_app_RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP_event(event);

            /* LED1 Blink */
            if (RBLE_OK == event->param.status)
            {
                LED2 = LED_OFF;
                led_blink_interval = LED_BLINK_ADV;
                RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)R_APP_LED1_Blink, led_blink_interval);
            }
            sw_usr_is_pushed = SW_RELEASED;
            break;
        case RBLE_GAP_EVENT_BROADCAST_DISABLE_COMP:
            break;
        case RBLE_GAP_EVENT_SET_BONDING_MODE_COMP:
            r_ble_app_RBLE_GAP_EVENT_SET_BONDING_MODE_COMP_event(event);
            break;
        case RBLE_GAP_EVENT_SET_SECURITY_REQUEST_COMP:
            break;
        case RBLE_GAP_EVENT_GET_DEVICE_INFO_COMP:
            break;
        case RBLE_GAP_EVENT_RPA_RESOLVED:
            break;
        case RBLE_GAP_EVENT_GET_WHITE_LIST_SIZE_COMP:
            break;
        case RBLE_GAP_EVENT_ADD_TO_WHITE_LIST_COMP:
            break;
        case RBLE_GAP_EVENT_DEL_FROM_WHITE_LIST_COMP:
            break;
        case RBLE_GAP_EVENT_GET_REMOTE_DEVICE_NAME_COMP:
            break;
        case RBLE_GAP_EVENT_GET_REMOTE_DEVICE_INFO_COMP:
            break;
        case RBLE_GAP_EVENT_DEVICE_SEARCH_COMP:
            break;
        case RBLE_GAP_EVENT_DEVICE_SEARCH_RESULT_IND:
            break;
        case RBLE_GAP_EVENT_SET_RANDOM_ADDRESS_COMP:
            break;
        case RBLE_GAP_EVENT_SET_PRIVACY_FEATURE_COMP:
            break;
        case RBLE_GAP_EVENT_CONNECTION_COMP:
            led_blink_interval = LED_BLINK_NORMAL;
            R_APP_Mode_Set(APP_MODE_BLE_PARAM_SETTING);
            r_ble_app_RBLE_GAP_EVENT_CONNECTION_COMP_event(event);
            break;
        case RBLE_GAP_EVENT_CONNECTION_CANCEL_COMP:
            break;
        case RBLE_GAP_EVENT_DISCONNECT_COMP:
            RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);
            LED1 = LED_OFF;
            LED2 = LED_OFF;
            r_ble_app_RBLE_GAP_EVENT_DISCONNECT_COMP_event(event);

            dataflash_w_lora_param_flag(&lpf);
            {
                uint16_t    i = 10000;
                while(i--);
            }

            if ((lpf.param & LORA_PARAM_FLAG_MASK) == LORA_PARAM_FLAG_MASK)
            {
                R_LORA_APP_Set_RunCmd(LORA_AT_SAVE_CMD);
                R_APP_Mode_Set(APP_MODE_LORA_SAVE_PARAM);
            }
            else
            {
                R_APP_Mode_Set(APP_MODE_IDLE);
            }
            break;
        case RBLE_GAP_EVENT_ADVERTISING_REPORT_IND:
            break;
        case RBLE_GAP_EVENT_BONDING_COMP:
            r_ble_app_RBLE_GAP_EVENT_BONDING_COMP_event(event);
            break;
        case RBLE_GAP_EVENT_BONDING_REQ_IND:
            r_ble_app_RBLE_GAP_EVENT_BONDING_REQ_IND_event(event);
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_REQ_IND:
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_COMP:
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_RESPONSE:
            break;
        case RBLE_GAP_EVENT_CHANNEL_MAP_REQ_COMP:
            break;
        case RBLE_GAP_EVENT_READ_RSSI_COMP:
            break;
        case RBLE_GAP_EVENT_WR_CHAR_IND:
            break;
        case RBLE_GAP_EVENT_COMMAND_DISALLOWED_IND:
            break;
        default:
            break;
    }
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_RESET_RESULT_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_RESET_RESULT
* Arguments    : event - GAP Event Parameters
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_RESET_RESULT_COMP_event(RBLE_GAP_EVENT *event)
{
    RBLE_STATUS     ret_status = FALSE;

    if (RBLE_OK == event->param.status)
    {
        ret_status = TRUE;
        
        /* Set Bonding Mode */
        R_BLE_APP_Set_RunCmd(GAP_SET_BONDING_MODE_CMD);
    }

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP
* Arguments    : event - GAP Event Parameters
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP_event(RBLE_GAP_EVENT *event)
{
    RBLE_STATUS     ret_status = FALSE;

    if (RBLE_OK == event->param.status)
    {
        ret_status = TRUE;
    }

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_CONNECTION_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_CONNECTION_COMP
* Arguments    : event - GAP Event Parameters
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_CONNECTION_COMP_event(RBLE_GAP_EVENT *event)
{
    RBLE_STATUS ret_status = FALSE;

    if (RBLE_OK == event->param.status)
    {
        ConnectionHdl  = event->param.conn_comp.connect_info.conhdl;
        RemoteDevice   = event->param.conn_comp.connect_info.peer_addr;
        FoundFlg       = FALSE;
        CheckedIrkFlg  = FALSE;
        ret_status = TRUE;
        
        /* Enable Profile (Profile is Disabled automatically when Disconnection) */
        R_BLE_APP_Set_RunCmd(GPCS_Server_Enable_CMD);
    }

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_DISCONNECT_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_DISCONNECT_COMP
* Arguments    : event - GAP Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_DISCONNECT_COMP_event(RBLE_GAP_EVENT *event)
{
    ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;

    /* Disable timer for indication trigger */
    RBLE_Clear_Timer_Id( _RBLE_TIMER_ID_APP );

    RBLE_VUART_Server_Disable();

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_SET_BONDING_MODE_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_SET_BONDING_MODE_COMP
* Arguments    : event - GAP Event Parameters
* Return Value : ret_status - RBLE_STATUS_enum
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_SET_BONDING_MODE_COMP_event(RBLE_GAP_EVENT *event)
{
    RBLE_STATUS ret_status = FALSE;

    if (RBLE_OK == event->param.status)
    {
        ret_status = TRUE;
        
        /* Set Security Request */
        R_BLE_APP_Set_RunCmd(GAP_SET_SECURITY_REQUEST_CMD);
    }

    return(ret_status);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_BONDING_COMP_event
* Description  : Event processing of RBLE_GAP_EVENT_BONDING_COMP
* Arguments    : event - GAP Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_BONDING_COMP_event(RBLE_GAP_EVENT *event)
{
    InBonding      = FALSE;
    SecStatus      = event->param.bonding_comp.sec_prop | RBLE_SMP_BONDED;
    BondedAddr     = RemoteDevice  ;

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_GAP_EVENT_BONDING_REQ_IND_event
* Description  : Event processing of RBLE_GAP_EVENT_BONDING_REQ_IND
* Arguments    : event - GAP Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_GAP_EVENT_BONDING_REQ_IND_event(RBLE_GAP_EVENT *event)
{
    InBonding = TRUE;

    /* Reset Remote Device Property */
    SecStatus      = RBLE_SMP_SEC_NONE;
    memset(&BondedAddr, 0, sizeof(RBLE_BD_ADDR));
    memset(&LocalKey  , 0, sizeof(APP_KEY_DATA));
    memset(&RemoteKey , 0, sizeof(APP_KEY_DATA));

    /* Reply Bonding Response */
    R_BLE_APP_Set_RunCmd(GAP_BONDING_RESPONSE_CMD);

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_sm_callback
* Description  : Callback function of Security Manager
* Arguments    : event - SM Event Parameters
* Return Value : void
******************************************************************************/
static void r_ble_app_sm_callback(RBLE_SM_EVENT *event)
{
    switch(event->type)
    {
        case RBLE_SM_EVENT_SET_CNF:
            break;
        case RBLE_SM_ENC_START_IND:
            break;
        case RBLE_SM_TK_REQ_IND:
            r_ble_app_RBLE_SM_TK_REQ_IND_event(event);
            break;
        case RBLE_SM_LTK_REQ_IND:
            r_ble_app_RBLE_SM_LTK_REQ_IND_event(event);
            break;
        case RBLE_SM_IRK_REQ_IND:
            r_ble_app_RBLE_SM_IRK_REQ_IND_event(event);
            break;
        case RBLE_SM_CSRK_REQ_IND:
            break;
        case RBLE_SM_KEY_IND:
            r_ble_app_RBLE_SM_KEY_IND_event(event);
            break;
        case RBLE_SM_CHK_BD_ADDR_REQ:
            r_ble_app_RBLE_SM_CHK_BD_ADDR_REQ_event(event);
            break;
        case RBLE_SM_TIMEOUT_EVT:
            break;
        case RBLE_SM_EVENT_COMMAND_DISALLOWED_IND:
            break;
        default:
            break;
    }
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_SM_TK_REQ_IND_event
* Description  : Event processing of RBLE_SM_TK_REQ_IND
* Arguments    : event - SM Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_SM_TK_REQ_IND_event(RBLE_SM_EVENT *event)
{
    ConnectionIdx = event->param.tk_req.idx;

    /* Reply TK(Temporary Key) */
    R_BLE_APP_Set_RunCmd(SM_TK_REQ_RESP_CMD);

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_SM_LTK_REQ_IND_event
* Description  : Event processing of RBLE_SM_LTK_REQ_IND
* Arguments    : event - SM Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_SM_LTK_REQ_IND_event(RBLE_SM_EVENT *event)
{
    ConnectionIdx = event->param.ltk_req.idx;

    /* Reply LTK(Long Term Key) */
    R_BLE_APP_Set_RunCmd(SM_LTK_REQ_RESP_CMD);

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_SM_IRK_REQ_IND_event
* Description  : Event processing of RBLE_SM_IRK_REQ_IND
* Arguments    : event - SM Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_SM_IRK_REQ_IND_event(RBLE_SM_EVENT *event)
{
    ConnectionIdx = event->param.irk_req.idx;

    /* Reply IRK(Identity Resolving Key) */
    R_BLE_APP_Set_RunCmd(SM_IRK_REQ_RESP_CMD);

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_SM_KEY_IND_event
* Description  : Event processing of RBLE_SM_KEY_IND
* Arguments    : event - SM Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_SM_KEY_IND_event(RBLE_SM_EVENT *event)
{
    /* Store Encryption Key */
    if(RBLE_SMP_KDIST_ENCKEY       == event->param.key_ind.key_code)
    {
        RemoteKey.EncKey         = event->param.key_ind;
        RemoteKey.Key_Set_Flags |= (1 << APP_ENCKEY );
    }
    else if(RBLE_SMP_KDIST_IDKEY   == event->param.key_ind.key_code)
    {
        RemoteKey.IrKey          = event->param.key_ind;
        RemoteKey.Key_Set_Flags |= (1 << APP_IDKEY  );
    }
    else if(RBLE_SMP_KDIST_SIGNKEY == event->param.key_ind.key_code)
    {
        RemoteKey.SignKey        = event->param.key_ind;
        RemoteKey.Key_Set_Flags |= (1 << APP_SIGNKEY);
    }

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_RBLE_SM_CHK_BD_ADDR_REQ_event
* Description  : Event processing of RBLE_SM_CHK_BD_ADDR_REQ
* Arguments    : event - SM Event Parameters
* Return Value : TRUE - succeeded
******************************************************************************/
static BOOL r_ble_app_RBLE_SM_CHK_BD_ADDR_REQ_event(RBLE_SM_EVENT *event)
{
    ConnectionIdx = event->param.chk_bdaddr.idx;
    
    /* Check if Remote Device is already bonded */
    FoundFlg = r_ble_app_compare_bdaddr(&BondedAddr, &event->param.chk_bdaddr.addr);
    
    /* Reply BD Address Check Result */
    R_BLE_APP_Set_RunCmd(SM_CHK_BD_ADDR_REQ_RESP_CMD);

    return(TRUE);
}

/******************************************************************************
* Function Name: r_ble_app_gpcs_callback
* Description  : Callback function of General Purpose Communication Server (VUART)
* Arguments    : void
* Return Value : void
******************************************************************************/
static void r_ble_app_gpcs_callback(RBLE_VUART_EVENT *event)
{
    switch(event->type)
    {
        case RBLE_VUART_EVENT_SERVER_ENABLE_CMP:
            break;
        case RBLE_VUART_EVENT_SERVER_WRITE_REQ:
            R_LORA_APP_ATCmd_Generate((int8_t *)&(event->param.server_write_req.value),
                                    event->param.server_write_req.len);
            R_LORA_APP_Set_RunCmd(LORA_AT_PARAM_SEND_CMD);
            break;
        case RBLE_VUART_EVENT_SERVER_INDICATION_CFM:
            break;
        default:
            break;
    }

    return;
}

/******************************************************************************
* Function Name: r_ble_app_compare_bdaddr
* Description  : Get random value
* Arguments    : a - BD Address
*                b - BD Address
* Return Value : TRUE  - BD address is same
*                FALSE - BD address is different
******************************************************************************/
static BOOL r_ble_app_compare_bdaddr(RBLE_BD_ADDR* a, RBLE_BD_ADDR* b)
{
    uint8_t i;

    for(i = 0; i < RBLE_BD_ADDR_LEN; i++)
    {
        if(a->addr[i] != b->addr[i])
        {
            return FALSE;
        }
    }
    return TRUE;
}

/******************************************************************************
* Function Name: r_ble_app_get_random
* Description  : Get random value
* Arguments    : void
* Return Value : void
******************************************************************************/
static int_t r_ble_app_get_random(void)
{
    return(rand());
}
