/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this
 * software and to discontinue the availability of this software. By using this
 * software, you agree to the additional terms and conditions found by
 * accessing the following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/*******************************************************************************
* File Name    : r_app_ble.c
* Version      : 1.0
* Description  : BLE controll application.
*                Connect with a smartphone to receive LoRa module setting parameters.
******************************************************************************/
/*****************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 31.05.2019 1.00     First Release
******************************************************************************/

#ifndef APP_RLIOT_NODE_H
#define APP_RLIOT_NODE_H

/******************************************************************************
Includes
******************************************************************************/

#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_serial.h"
#include "r_cg_timer.h"
#include "r_cg_it.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/******************************************************************************
Typedef definitions
******************************************************************************/

enum LORA_APP_CMD_enum {
    LORA_CMD_NON,
    LORA_AT_ECHO_DI_CMD,
    LORA_AT_PARAM_SEND_CMD,
    LORA_AT_RESP_SEND_CMD,
    LORA_AT_SAVE_CMD,
    LORA_AT_PARAM_WRITE_CMD,
    LORA_AT_JOIN_CMD,
    LORA_AT_SENDHEX_CMD,
};

enum
{
    APP_LORA_PARAM_NG,
    APP_LORA_PARAM_OK
};

enum
{
    APP_MODE_INITIALIZATION,        /* 0 */
    APP_MODE_IDLE,                  /* 1 */
    APP_MODE_BLE_ADVERTISING,       /* 2 */
    APP_MODE_BLE_PARAM_SETTING,     /* 3 */
    APP_MODE_LORA_SAVE_PARAM,       /* 4 */
    APP_MODE_LORA_JOIN,             /* 5 */
    APP_MODE_LORA_COMMUNICATION,    /* 6 */
};

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

void R_LORA_APP_Init(void);
void R_LORA_APP_Run(void);
void R_LORA_APP_Set_RunCmd(uint16_t cmd);
void R_LORA_APP_ATCmd_Generate(int8_t *value, uint8_t len);
void R_LORA_APP_ATResult_Generate(uint8_t c);
void R_LORA_APP_Wait_Timer_Callback(void);

#endif // APP_RLIOT_NODE_H
