/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_main.c
* Version      : CodeGenerator for RL78/G14 V2.05.03.02 [06 Nov 2018]
* Device(s)    : R5F104ML
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 2019/11/20
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_intc.h"
#include "r_cg_serial.h"
#include "r_cg_timer.h"
#include "r_cg_it.h"
/* Start user code for include. Do not edit comment generated here */
#include "arch.h"
#include "rscip_api.h"
#include "timer.h"
#include "rble_app.h"
#include "rble_host.h"
#include "r_uart_ble_modem.h"
#include "r_ms8607.h"
#include "r_app_ms8607.h"

#include "dataflash.h"
#include "r_app_lora.h"
#include "r_uart_lora.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
uint8_t app_mode;

uint8_t sw_usr_pushed_count = 0;
uint8_t sw_usr_is_pushed    = SW_RELEASED;

int16_t app_wait_ms_timer_count;    /* TAU0 timer count */
int16_t led_blink_interval;

extern LoRa_Param_Flag_t   lpf;

extern void R_BLE_APP_Set_RunCmd(uint16_t cmd);
/* End user code. Do not edit comment generated here */
void R_MAIN_UserInit(void);

/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{
    R_MAIN_UserInit();
    /* Start user code. Do not edit comment generated here */

    LED1 = LED_OFF;
    LED2 = LED_OFF;

    /* Initialize FDL(Data Flash Library) and EEL(EEPROM Emuration Library) */
    dataflash_init();

    if (PORT_SW_USR == 0)
    {
        dataflash_erase_param();
    }

    /* Rad LoRaWAN(TM) module parameters from dataflash */
    dataflash_read_param();

    /* RY7011: UART 2-wire with branch connection */
    PORT_RY7011_UART_2WIRE_BRANCH = 0;

    /* Deassert RESET# */
    PORT_RY7011_RESET = 1;
    PORT_LORA_RESET   = 1;

    EI();

    /* Deassert reset wait */
    R_APP_Wait_Timer(1000);

    /* Initialize LoRaWAN application */
    R_LORA_APP_Init();

    /* Initialize sensor */
    R_MS8607_APP_Init();

    /* Push switch(SW_USR): Initialize interruption(INTC0(P137)) */
    R_INTC0_Start();

    /* Initialize rBLE Timer */
    if ( RBLE_OK == RBLE_Init_Timer() )
    {
        /* Intialize Application */
        if ( TRUE == R_BLE_APP_Init() )
        {
            /* Main Loop(not break) */
            while(1)
            {
                /* BLE processing of between the RL78/G1D and a smartphone */
                /* for setting of LoRa modem parameters */
                R_BLE_APP_Run();
                rBLE_Run();

                /* LoRa modem processing */
                /* for LoRa communication */
                R_LORA_APP_Run();
                
                /* MCU Stop Management Section */
                GLOBAL_INT_DISABLE();
                if(rBLE_Is_Idle() && R_BLE_APP_Is_Idle() && RSCIP_Can_Sleep()
                   && rBLE_Timer_Can_Sleep() && app_uart2_can_sleep())
                {
                    __halt();
                }
                GLOBAL_INT_RESTORE();
            }
        }
    }

    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/******************************************************************************
* Function Name: R_APP_Mode_Set
* Description  : Set application mode
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_APP_Mode_Set(uint8_t mode)
{
    app_mode = mode;
}

/******************************************************************************
* Function Name: R_APP_LED1_Blink
* Description  : LED1 blink
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_APP_LED1_Blink(void)
{
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)R_APP_LED1_Blink, led_blink_interval);

    LED1 ^= 1;

    return;
}

/******************************************************************************
* Function Name: R_APP_LED2_Blink
* Description  : LED2 blink
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_APP_LED2_Blink(void)
{
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)R_APP_LED2_Blink, led_blink_interval);

    LED2 ^= 1;

    return;
}

/******************************************************************************
* Function Name: R_BLE_APP_Sw_Usr
* Description  : Switch control function on FPB board.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_APP_Sw_Usr(void)
{
    if (app_mode == APP_MODE_IDLE)
    {
        if (PORT_SW_USR == 0)
        {
            /* Long push processing (execute AT+JOIN) */

            sw_usr_pushed_count++;
            if (sw_usr_pushed_count < SWITCH_LONG_PUSHED_COUNT)
            {
                /* Push count did not expire */

                RBLE_Set_Timer_Id(_RBLE_TIMER_ID_SW, (RBLE_FUNC)R_APP_Sw_Usr, SWITCH_CHECK_INTERVAL);
            }
            else
            if ((lpf.param & LORA_PARAM_FLAG_MASK) == LORA_PARAM_FLAG_MASK)
            {
                /* Push count expired */
                
                /* All LoRa module parameters are setted */
                /* and send "AT+JOIN" command to LoRa module */
                R_LORA_APP_Set_RunCmd(LORA_AT_JOIN_CMD);
                R_APP_Mode_Set(APP_MODE_LORA_JOIN);
            }
            else
            {
                sw_usr_pushed_count = 0;
                sw_usr_is_pushed = SW_RELEASED;
            }
        }
        else
        {
            /* Short push processing (execute Advertising) */

            R_APP_Mode_Set(APP_MODE_BLE_ADVERTISING);
            R_LORA_APP_Set_RunCmd(LORA_CMD_NON);

            RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_SEND_SENSOR_DATA);
            RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);

            LED2 = LED_OFF;
            
            /* Start advertising */
            R_BLE_APP_Set_RunCmd(GAP_BROADCAST_ENABLE_CMD);
        }
    }
    else
    {
        /* BLE advertising is executing. */
        /* LoRaWAN transmission is executing.  */

        if (app_mode == APP_MODE_BLE_PARAM_SETTING)
        {
            /* Do nothing when BLE is connected. */
            return;
        }

        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_SEND_SENSOR_DATA);
        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);

        LED1 = LED_OFF;
        LED2 = LED_OFF;

        if (app_mode == APP_MODE_BLE_ADVERTISING)
        {
            R_BLE_APP_Set_RunCmd(GAP_BROADCAST_DISABLE_CMD);
        }

        R_APP_Mode_Set(APP_MODE_IDLE);
        sw_usr_is_pushed = SW_RELEASED;
    }
}

/******************************************************************************
* Function Name: R_APP_Wait_Timer
* Description  : Wait function.
* Arguments    : msec - wait count
* Return Value : void
******************************************************************************/
void R_APP_Wait_Timer(uint16_t msec)
{
    app_wait_ms_timer_count = msec;
    R_TAU0_Channel0_Start();
    while(app_wait_ms_timer_count > 0);
    R_TAU0_Channel0_Stop();
}

/******************************************************************************
* Function Name: R_APP_Wait_Timer_Callback
* Description  : This function count TAU interruption.
*                It is called from r_tau0_channel0_interrupt().
* Arguments    : void
* Return Value : extern int16_t wait_timer_count;
******************************************************************************/
void R_APP_Wait_Timer_Callback(void)
{
    app_wait_ms_timer_count--;
}
/* End user code. Do not edit comment generated here */
